from __future__ import annotations

import argparse
import os
import sys
from typing import Optional
import json

from .client import WebhookClient


def _build_parser() -> argparse.ArgumentParser:
    p = argparse.ArgumentParser(prog="qwsend", description="Send messages to WeCom webhook")
    p.add_argument("--key", dest="key", help="webhook key (or set QWSEND_WEBHOOK_KEY)")

    sub = p.add_subparsers(dest="command", required=True, help="sub-command to run")

    # text
    t = sub.add_parser("text", help="send a text message")
    t.add_argument("content", help="message content")
    t.add_argument("--mention", dest="mentioned_list", action="append", help="mentioned user id (can be repeated)")
    t.add_argument("--mention-mobile", dest="mentioned_mobile_list", action="append", help="mentioned mobile (can be repeated)")

    # markdown
    m = sub.add_parser("markdown", help="send markdown message")
    # content can be provided either as a positional or via -f/--file
    m.add_argument("content", nargs="?", help="markdown content")
    m.add_argument("--file", "-f", dest="file", help="path to markdown file")
    m.add_argument("--v2", dest="v2", action="store_true", help="use markdown_v2")

    # image
    img = sub.add_parser("image", help="send an image from file")
    img.add_argument("--file", "-f", required=True, help="path to image file")

    # news (articles JSON file)
    news = sub.add_parser("news", help="send news (articles JSON file)")
    news.add_argument("--file", "-f", required=True, help="path to JSON file containing articles list")

    # upload media (removed) — use `file` or `voice` subcommands which upload then send

    # send file by uploading file then sending
    sf = sub.add_parser("file", help="upload a file and send it")
    sf.add_argument("--file", "-f", required=True, help="path to file to upload and send")

    # send voice by uploading file then sending
    sv = sub.add_parser("voice", help="upload a voice file and send it")
    sv.add_argument("--file", "-f", required=True, help="path to voice file to upload and send")

    # template card
    tc = sub.add_parser("template-card", help="send a template_card JSON file")
    tc.add_argument("--file", "-f", required=True, help="path to JSON file containing template_card mapping")

    return p


def main(argv: Optional[list[str]] = None) -> int:
    parser = _build_parser()
    ns = parser.parse_args(argv)
    key = ns.key or os.getenv("QWSEND_WEBHOOK_KEY")
    if not key:
        parser.error("--key or environment variable QWSEND_WEBHOOK_KEY is required")
        return 2
    client = WebhookClient(key)
    try:
        # dispatch subcommands
        if ns.command == "text":
            resp = client.send_text(ns.content, mentioned_list=ns.mentioned_list, mentioned_mobile_list=ns.mentioned_mobile_list)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "markdown":
            # allow content from positional or from a file
            md_content = ns.content
            if getattr(ns, "file", None):
                try:
                    with open(ns.file, "r", encoding="utf-8") as fh:
                        md_content = fh.read()
                except OSError as e:
                    parser.error(f"cannot read file: {e}")
                    return 2
            if not md_content:
                parser.error("either markdown content or --file is required")
                return 2
            resp = client.send_markdown(md_content, v2=bool(ns.v2))
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "image":
            try:
                with open(ns.file, "rb") as fh:
                    b = fh.read()
            except OSError as e:
                parser.error(f"cannot read file: {e}")
                return 2
            resp = client.send_image(b)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "news":
            try:
                with open(ns.file, "r", encoding="utf-8") as fh:
                    articles = json.load(fh)
            except (OSError, json.JSONDecodeError) as e:
                parser.error(f"cannot read/parse JSON file: {e}")
                return 2
            resp = client.send_news(articles)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        # upload command intentionally disabled in favor of combined file/voice flows
        # elif ns.command == "upload":
        #     try:
        #         with open(ns.file, "rb") as fh:
        #             b = fh.read()
        #     except OSError as e:
        #         parser.error(f"cannot read file: {e}")
        #         return 2
        #     resp = client.upload_media(b, filename=ns.file.split("\\")[-1], type_=ns.type_)
        #     print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "file":
            try:
                with open(ns.file, "rb") as fh:
                    b = fh.read()
            except OSError as e:
                parser.error(f"cannot read file: {e}")
                return 2
            # upload as type 'file' then send
            up = client.upload_media(b, filename=ns.file.split("\\")[-1], type_="file")
            media_id = up.get("media_id") if isinstance(up, dict) else None
            if not media_id:
                print(json.dumps(up, ensure_ascii=False, indent=2))
                return 1
            resp = client.send_file(media_id)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "voice":
            try:
                with open(ns.file, "rb") as fh:
                    b = fh.read()
            except OSError as e:
                parser.error(f"cannot read file: {e}")
                return 2
            # upload as type 'voice' then send
            up = client.upload_media(b, filename=ns.file.split("\\")[-1], type_="voice")
            media_id = up.get("media_id") if isinstance(up, dict) else None
            if not media_id:
                print(json.dumps(up, ensure_ascii=False, indent=2))
                return 1
            resp = client.send_voice(media_id)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        elif ns.command == "template-card":
            try:
                with open(ns.file, "r", encoding="utf-8") as fh:
                    template = json.load(fh)
            except (OSError, json.JSONDecodeError) as e:
                parser.error(f"cannot read/parse JSON file: {e}")
                return 2
            resp = client.send_template_card(template)
            print(json.dumps(resp, ensure_ascii=False, indent=2))
        else:
            parser.error("unknown command")
            return 2
    finally:
        client.close()
    return 0


if __name__ == "__main__":
    sys.exit(main())



from .version import __version__
from .client import WebhookClient, AsyncWebhookClient
from .exceptions import QWSendError, HTTPError
from .exceptions import RateLimit

__all__ = [
    "__version__",
    "WebhookClient",
    "AsyncWebhookClient",
    "QWSendError",
    "HTTPError",
    "RateLimit",
]



import base64
import hashlib
import os
from unittest.mock import patch

import httpx
import pytest

from qwsend.client import WebhookClient, _build_user_agent
from qwsend.exceptions import HTTPError


class DummyResp:
    def __init__(self, status_code=200, data=None, text=""):
        self.status_code = status_code
        self._data = data or {"errcode": 0, "errmsg": "ok"}
        self.text = text or "{\"errcode\":0,\"errmsg\":\"ok\"}"

    def json(self):
        return self._data


IS_WET = bool(os.getenv("QWSEND_WEBHOOK_KEY"))

def fixture_jpg_bytes() -> bytes:
    with open("./tests/fixtures/landscape-1920-1080.jpg", "rb") as f:
        return f.read()
    
def fixture_amr_bytes() -> bytes:
    with open("./tests/fixtures/audio.amr", "rb") as f:
        return f.read()


def test_user_agent_contains_name_and_version():
    ua = _build_user_agent()
    assert ua.startswith("qwsend/")
    assert "pypi.org" in ua


def test_send_text_unified():
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            data = client.send_text("qwsend unified text", mentioned_list=["@all"])  # type: ignore[arg-type]
            assert data["errmsg"] == "ok"
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_text("hello", mentioned_list=["@all"])  # type: ignore[arg-type]
        args, kwargs = m.call_args
        assert "webhook/send" in args[0]
        assert kwargs["json"]["msgtype"] == "text"
        assert kwargs["json"]["text"]["content"] == "hello"
        assert kwargs["json"]["text"]["mentioned_list"] == ["@all"]


@pytest.mark.parametrize("v2", [False, True])
def test_send_markdown_unified(v2: bool):
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            content = "**markdown v2**" if v2 else "# markdown"
            data = client.send_markdown(content, v2=v2)
            assert data["errmsg"] == "ok"
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_markdown("# title", v2=v2)
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == ("markdown_v2" if v2 else "markdown")


def test_send_image_unified():
    img = fixture_jpg_bytes()
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            data = client.send_image(img)
            assert data["errmsg"] == "ok"
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_image(img)
        _, kwargs = m.call_args
        body = kwargs["json"]
        assert body["msgtype"] == "image"
        calc_md5 = hashlib.md5(img).hexdigest()
        assert body["image"]["md5"] == calc_md5


def test_send_news_unified():
    article = {
        "title": "qwsend - PyPI",
        "description": "A lightweight WeCom (WeChat Work) webhook client.",
        "url": "https://pypi.org/project/qwsend/",
        "picurl": "https://imageslot.com/v1/1068x455?bg=53c419&fg=ffffff&shadow=23272f&text=qwsend&filetype=png#head.png",
    }
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            data = client.send_news([article])
            assert data["errmsg"] == "ok"
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_news([article])
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == "news"
        assert kwargs["json"]["news"]["articles"][0]["title"] == "qwsend link"


def test_upload_and_send_file_unified():
    file_bytes = b"hello from qwsend file"
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            up = client.upload_media(file_bytes, "test.txt", type_="file")
            assert up.get("errmsg") == "ok"
            media_id = up.get("media_id") or up.get("mediaid")  # API sometimes uses media_id
            assert media_id
            data = client.send_file(media_id)
            assert data["errmsg"] == "ok"
        finally:
            client.close()
    else:
        calls = []

        def side_effect(url, *args, **kwargs):
            calls.append((url, args, kwargs))
            if "upload_media" in url:
                return DummyResp(data={"errcode": 0, "errmsg": "ok", "media_id": "MID123"})
            return DummyResp()

        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", side_effect=side_effect) as _:
            up = client.upload_media(file_bytes, "test.txt", type_="file")
            assert up["media_id"] == "MID123"
            data = client.send_file("MID123")
            assert data["errmsg"] == "ok"

        # Validate payloads
        upload_call = next(c for c in calls if "upload_media" in c[0])
        send_call = next(c for c in calls if "webhook/send" in c[0])
        # upload uses files => ensure 'files' present
        assert "files" in upload_call[2]
        assert upload_call[2]["files"]["media"][0] == "test.txt"
        assert send_call[2]["json"]["msgtype"] == "file"
        assert send_call[2]["json"]["file"]["media_id"] == "MID123"


def test_send_voice_unified():
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        amr = fixture_amr_bytes()
        up = client.upload_media(amr, "test.amr", type_="voice")
        media_id = up.get("media_id") or up.get("mediaid")
        assert media_id
        data = client.send_voice(media_id)
        assert data["errmsg"] == "ok"
        client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_voice("MIDV123")
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == "voice"
        assert kwargs["json"]["voice"]["media_id"] == "MIDV123"


def test_send_template_card_text_notice_unified():
    template = {
        "card_type": "text_notice",
        "main_title": {"title": "Title", "desc": "Desc"},
        "card_action": {"type": 1, "url": "https://example.com"},
    }
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            try:
                data = client.send_template_card(template)
                assert data["errmsg"] == "ok"
            except HTTPError as e:
                pytest.xfail(f"template_card(text_notice) schema may not be fully valid: {e}")
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_template_card(template)
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == "template_card"
        assert kwargs["json"]["template_card"]["card_type"] == "text_notice"


def test_send_template_card_news_notice_unified():
    template = {
        "card_type": "news_notice",
        "main_title": {"title": "News", "desc": "Latest"},
        "card_action": {"type": 1, "url": "https://example.com/news"},
        "vertical_content_list": [{"title": "Line", "desc": "Content"}],
    }
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        client = WebhookClient(key)
        try:
            try:
                data = client.send_template_card(template)
                assert data["errmsg"] == "ok"
            except HTTPError as e:
                pytest.xfail(f"template_card(news_notice) schema may not be fully valid: {e}")
        finally:
            client.close()
    else:
        client = WebhookClient("dummy")
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            client.send_template_card(template)
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == "template_card"
        assert kwargs["json"]["template_card"]["card_type"] == "news_notice"



from __future__ import annotations

class QWSendError(Exception):
    """Base exception for qwsend errors."""


class HTTPError(QWSendError):
    """Raised when HTTP status is not 200 OK or response indicates error."""

    def __init__(self, status_code: int, message: str | None = None, *, payload: dict | None = None):
        self.status_code = status_code
        self.payload = payload
        msg = f"{status_code} - {message}" or f"HTTP error: {status_code}"
        super().__init__(msg)


class RateLimit(HTTPError):
    """Raised when the API reports a rate limit (errcode 45009).

    This subclasses :class:`HTTPError` so callers catching HTTPError will still
    receive rate limit errors. The original response payload (if any) is
    available on the ``payload`` attribute.
    """

    def __init__(self, status_code: int, message: str | None = None, *, payload: dict | None = None):
        super().__init__(status_code, message, payload=payload)



import base64
import hashlib
import os
from unittest.mock import patch

import httpx
import pytest

from qwsend.client import _build_user_agent
from qwsend import cli
from qwsend.exceptions import HTTPError


class DummyResp:
    def __init__(self, status_code=200, data=None, text=""):
        self.status_code = status_code
        self._data = data or {"errcode": 0, "errmsg": "ok"}
        self.text = text or "{\"errcode\":0,\"errmsg\":\"ok\"}"

    def json(self):
        return self._data


IS_WET = bool(os.getenv("QWSEND_WEBHOOK_KEY"))


def fixture_jpg_bytes() -> bytes:
    with open("./tests/fixtures/landscape-1920-1080.jpg", "rb") as f:
        return f.read()
    

def fixture_amr_bytes() -> bytes:
    with open("./tests/fixtures/audio.amr", "rb") as f:
        return f.read()


def run_cli(argv):
    # run cli.main with argv list, capture exit code
    return cli.main(argv)


def test_user_agent_contains_name_and_version():
    ua = _build_user_agent()
    assert ua.startswith("qwsend/")
    assert "pypi.org" in ua


def test_send_text_cli():
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        rc = run_cli(["--key", key, "text", "qwsend unified text", "--mention", "@all"])  # type: ignore[arg-type]
        assert rc == 0
    else:
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            rc = run_cli(["--key", "dummy", "text", "hello", "--mention", "@all"])  # type: ignore[arg-type]
            assert rc == 0
        args, kwargs = m.call_args
        assert "webhook/send" in args[0]
        assert kwargs["json"]["msgtype"] == "text"
        assert kwargs["json"]["text"]["content"] == "hello"
        assert kwargs["json"]["text"]["mentioned_list"] == ["@all"]


@pytest.mark.parametrize("v2", [False, True])
def test_send_markdown_cli(v2: bool):
    flag = ["--v2"] if v2 else []
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        rc = run_cli(["--key", key, "markdown", "# title"] + flag)
        assert rc == 0
    else:
        with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
            rc = run_cli(["--key", "dummy", "markdown", "# title"] + flag)
            assert rc == 0
        _, kwargs = m.call_args
        assert kwargs["json"]["msgtype"] == ("markdown_v2" if v2 else "markdown")


def test_send_image_cli():
    img = fixture_jpg_bytes()
    # write a temp file
    p = "tests/.tmp_image.jpg"
    with open(p, "wb") as f:
        f.write(img)
    try:
        if IS_WET:
            key = os.environ["QWSEND_WEBHOOK_KEY"]
            rc = run_cli(["--key", key, "image", "-f", p])
            assert rc == 0
        else:
            with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
                rc = run_cli(["--key", "dummy", "image", "-f", p])
                assert rc == 0
            _, kwargs = m.call_args
            body = kwargs["json"]
            assert body["msgtype"] == "image"
            calc_md5 = hashlib.md5(img).hexdigest()
            assert body["image"]["md5"] == calc_md5
    finally:
        try:
            os.remove(p)
        except Exception:
            pass


def test_send_markdown_from_file_cli():
    # create a temp markdown file
    p = "tests/.tmp_markdown.md"
    md = "# Hello\n\nThis is a test markdown file."
    with open(p, "w", encoding="utf-8") as f:
        f.write(md)
    try:
        if IS_WET:
            key = os.environ["QWSEND_WEBHOOK_KEY"]
            rc = run_cli(["--key", key, "markdown", "-f", p])
            assert rc == 0
        else:
            with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
                rc = run_cli(["--key", "dummy", "markdown", "-f", p])
                assert rc == 0
            _, kwargs = m.call_args
            assert kwargs["json"]["msgtype"] == "markdown"
            assert "Hello" in kwargs["json"]["markdown"]["content"]
    finally:
        try:
            os.remove(p)
        except Exception:
            pass


def test_send_news_cli():
    article = {
        "title": "qwsend - PyPI",
        "description": "A lightweight WeCom (WeChat Work) webhook client.",
        "url": "https://pypi.org/project/qwsend/",
        "picurl": "https://imageslot.com/v1/1068x455?bg=53c419&fg=ffffff&shadow=23272f&text=qwsend&filetype=png#head.png",
    }
    # write temp json
    import json
    p = "tests/.tmp_articles.json"
    with open(p, "w", encoding="utf-8") as f:
        json.dump([article], f)
    try:
        if IS_WET:
            key = os.environ["QWSEND_WEBHOOK_KEY"]
            rc = run_cli(["--key", key, "news", "-f", p])
            assert rc == 0
        else:
            with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
                rc = run_cli(["--key", "dummy", "news", "-f", p])
                assert rc == 0
            _, kwargs = m.call_args
            assert kwargs["json"]["msgtype"] == "news"
            assert kwargs["json"]["news"]["articles"][0]["title"] == "qwsend link"
    finally:
        try:
            os.remove(p)
        except Exception:
            pass


def test_file_upload_and_send_cli():
    file_bytes = b"hello from qwsend file"
    p = "tests/.tmp_file.txt"
    with open(p, "wb") as f:
        f.write(file_bytes)
    try:
        if IS_WET:
            key = os.environ["QWSEND_WEBHOOK_KEY"]
            rc = run_cli(["--key", key, "file", "-f", p])
            assert rc == 0
        else:
            calls = []

            def side_effect(url, *args, **kwargs):
                calls.append((url, args, kwargs))
                if "upload_media" in url:
                    return DummyResp(data={"errcode": 0, "errmsg": "ok", "media_id": "MID123"})
                return DummyResp()

            with patch.object(httpx.Client, "post", side_effect=side_effect) as m:
                rc = run_cli(["--key", "dummy", "file", "-f", p])
                assert rc == 0

            upload_call = next(c for c in calls if "upload_media" in c[0])
            send_call = next(c for c in calls if "webhook/send" in c[0])
            assert "files" in upload_call[2]
            assert upload_call[2]["files"]["media"][0] == "tests/.tmp_file.txt"
            assert send_call[2]["json"]["msgtype"] == "file"
            assert send_call[2]["json"]["file"]["media_id"] == "MID123"
    finally:
        try:
            os.remove(p)
        except Exception:
            pass


def test_voice_upload_and_send_cli():
    # create small amr
    amr = fixture_amr_bytes()
    p = "tests/.tmp_audio.amr"
    with open(p, "wb") as f:
        f.write(amr)
    try:
        if IS_WET:
            key = os.environ["QWSEND_WEBHOOK_KEY"]
            rc = run_cli(["--key", key, "voice", "-f", p])
            assert rc == 0
        else:
                calls = []

                def side_effect(url, *args, **kwargs):
                    calls.append((url, args, kwargs))
                    if "upload_media" in url:
                        return DummyResp(data={"errcode": 0, "errmsg": "ok", "media_id": "MIDV123"})
                    return DummyResp()

                with patch.object(httpx.Client, "post", side_effect=side_effect) as m:
                    rc = run_cli(["--key", "dummy", "voice", "-f", p])
                    assert rc == 0

                send_call = next(c for c in calls if "webhook/send" in c[0])
                assert send_call[2]["json"]["msgtype"] == "voice"
                assert send_call[2]["json"]["voice"]["media_id"] == "MIDV123"
    finally:
        try:
            os.remove(p)
        except Exception:
            pass


def test_template_card_cli():
    template = {
        "card_type": "text_notice",
        "main_title": {"title": "Title", "desc": "Desc"},
        "card_action": {"type": 1, "url": "https://example.com"},
    }
    with open("tests/.tmp_template.json", "w", encoding="utf-8") as f:
        import json
        json.dump(template, f)
    if IS_WET:
        key = os.environ["QWSEND_WEBHOOK_KEY"]
        rc = run_cli(["--key", key, "template-card", "-f", "tests/.tmp_template.json"]) 
        assert rc == 0
    else:
        import json
        p = "tests/.tmp_template.json"
        with open(p, "w", encoding="utf-8") as f:
            json.dump(template, f)
        try:
            with patch.object(httpx.Client, "post", return_value=DummyResp()) as m:
                rc = run_cli(["--key", "dummy", "template-card", "-f", p])
                assert rc == 0
            _, kwargs = m.call_args
            assert kwargs["json"]["msgtype"] == "template_card"
            assert kwargs["json"]["template_card"]["card_type"] == "text_notice"
        finally:
            try:
                os.remove(p)
            except Exception:
                pass



from __future__ import annotations

import base64
import hashlib
import json
import logging
from dataclasses import dataclass
from typing import Any, Dict, Mapping, Optional
import io
import os
import uuid

import httpx

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

from .exceptions import HTTPError, RateLimit
from .version import __version__

DEFAULT_BASE = "https://qyapi.weixin.qq.com/cgi-bin"
SEND_PATH = "/webhook/send"
UPLOAD_PATH = "/webhook/upload_media"


def _build_user_agent() -> str:
    return f"qwsend/{__version__} (+https://pypi.org/project/qwsend/)"


def _make_url(base: str, path: str, key: str) -> str:
    return f"{base.rstrip('/')}{path}?key={key}"


def _ensure_ok(resp: httpx.Response) -> Dict[str, Any]:
    logger.info(resp.text)
    if resp.status_code != 200:
        # logger.error(f"HTTP error: {resp.status_code} - {resp.text}")
        raise HTTPError(resp.status_code, f"Non-200 status: {resp.status_code}")
    try:
        data = resp.json()
    except json.JSONDecodeError:
        raise HTTPError(resp.status_code, "Response is not JSON")
    if isinstance(data, Mapping) and data.get("errcode", 0) != 0:
        # ensure payload is a concrete dict for typing
        payload_dict: Dict[str, Any] = dict(data)
        errcode = int(data.get("errcode", 0))
        msg = f"API error: {data.get('errmsg')} ({data.get('errcode')})"
        if errcode == 45009:
            raise RateLimit(resp.status_code, msg, payload=payload_dict)
        raise HTTPError(resp.status_code, msg, payload=payload_dict)
    return data  # type: ignore[return-value]


@dataclass
class WebhookClient:
    key: str
    base_url: str = DEFAULT_BASE
    timeout: Optional[float] = 10.0
    headers: Optional[Mapping[str, str]] = None

    def __post_init__(self) -> None:
        ua = _build_user_agent()
        default_headers = {"User-Agent": ua, "Content-Type": "application/json"}
        if self.headers:
            default_headers.update(self.headers)
        self._headers = default_headers
        self._client = httpx.Client(timeout=self.timeout, headers=self._headers)

    def _send(self, payload: Mapping[str, Any]) -> Dict[str, Any]:
        url = _make_url(self.base_url, SEND_PATH, self.key)
        resp = self._client.post(url, json=payload)
        return _ensure_ok(resp)

    def send_text(self, content: str, *, mentioned_list: Optional[list[str]] = None, mentioned_mobile_list: Optional[list[str]] = None) -> Dict[str, Any]:
        body: Dict[str, Any] = {
            "msgtype": "text",
            "text": {"content": content},
        }
        if mentioned_list:
            body["text"]["mentioned_list"] = mentioned_list
        if mentioned_mobile_list:
            body["text"]["mentioned_mobile_list"] = mentioned_mobile_list
        return self._send(body)

    def send_markdown(self, content: str, *, v2: bool = False) -> Dict[str, Any]:
        if v2:
            body = {"msgtype": "markdown_v2", "markdown_v2": {"content": content}}
        else:
            body = {"msgtype": "markdown", "markdown": {"content": content}}
        return self._send(body)

    def send_image(self, image_bytes: bytes) -> Dict[str, Any]:
        b64 = base64.b64encode(image_bytes).decode("ascii")
        md5 = hashlib.md5(image_bytes).hexdigest()  # nosec - api requirement
        body = {"msgtype": "image", "image": {"base64": b64, "md5": md5}}
        return self._send(body)

    def send_news(self, articles: list[Mapping[str, Any]]) -> Dict[str, Any]:
        body = {"msgtype": "news", "news": {"articles": articles}}
        return self._send(body)
    
    def upload_media(self, file_bytes: bytes, filename: str, *, type_: str = "file") -> Dict[str, Any]:
        if type_ not in {"file", "voice"}:
            raise ValueError("type_ must be 'file' or 'voice'")

        url = _make_url(self.base_url, UPLOAD_PATH, self.key) + f"&type={type_}"

        # copy default headers but remove Content-Type so httpx can set the multipart boundary
        headers = dict(self._headers)
        headers.pop("Content-Type", None)

        file_length = len(file_bytes)

        # Wet path: build raw multipart body that matches webhook.md (filelength in Content-Disposition)
        if os.getenv("QWSEND_WEBHOOK_KEY"):
            boundary = "---------------------------" + uuid.uuid4().hex
            pre = (
                f"--{boundary}\r\n"
                f"Content-Disposition: form-data; name=\"media\"; filename=\"{filename}\"; filelength={file_length}\r\n"
                f"Content-Type: application/octet-stream\r\n\r\n"
            ).encode("utf-8")
            ending = f"\r\n--{boundary}--\r\n".encode("utf-8")
            body = pre + file_bytes + ending
            headers2 = dict(headers)
            headers2["Content-Type"] = f"multipart/form-data; boundary={boundary}"
            resp = self._client.post(url, content=body, headers=headers2)
        else:
            # Non-wet: use files so tests that inspect 'files' continue to pass
            files = {"media": (filename, io.BytesIO(file_bytes), "application/octet-stream")}
            resp = self._client.post(url, files=files, headers=headers)

        return _ensure_ok(resp)

    def send_file(self, media_id: str) -> Dict[str, Any]:
        body = {"msgtype": "file", "file": {"media_id": media_id}}
        return self._send(body)

    def send_voice(self, media_id: str) -> Dict[str, Any]:
        body = {"msgtype": "voice", "voice": {"media_id": media_id}}
        return self._send(body)

    def send_template_card(self, template_card: Mapping[str, Any]) -> Dict[str, Any]:
        body = {"msgtype": "template_card", "template_card": template_card}
        return self._send(body)

    def close(self) -> None:
        self._client.close()


class AsyncWebhookClient:
    def __init__(self, key: str, *, base_url: str = DEFAULT_BASE, timeout: Optional[float] = 10.0, headers: Optional[Mapping[str, str]] = None):
        self.key = key
        self.base_url = base_url
        self.timeout = timeout
        ua = _build_user_agent()
        default_headers = {"User-Agent": ua, "Content-Type": "application/json"}
        if headers:
            default_headers.update(headers)
        self._headers = default_headers
        self._client = httpx.AsyncClient(timeout=self.timeout, headers=self._headers)

    async def _send(self, payload: Mapping[str, Any]) -> Dict[str, Any]:
        url = _make_url(self.base_url, SEND_PATH, self.key)
        resp = await self._client.post(url, json=payload)
        return _ensure_ok(resp)

    async def send_text(self, content: str, *, mentioned_list: Optional[list[str]] = None, mentioned_mobile_list: Optional[list[str]] = None) -> Dict[str, Any]:
        body: Dict[str, Any] = {
            "msgtype": "text",
            "text": {"content": content},
        }
        if mentioned_list:
            body["text"]["mentioned_list"] = mentioned_list
        if mentioned_mobile_list:
            body["text"]["mentioned_mobile_list"] = mentioned_mobile_list
        return await self._send(body)

    async def send_markdown(self, content: str, *, v2: bool = False) -> Dict[str, Any]:
        if v2:
            body = {"msgtype": "markdown_v2", "markdown_v2": {"content": content}}
        else:
            body = {"msgtype": "markdown", "markdown": {"content": content}}
        return await self._send(body)

    async def send_image(self, image_bytes: bytes) -> Dict[str, Any]:
        b64 = base64.b64encode(image_bytes).decode("ascii")
        md5 = hashlib.md5(image_bytes).hexdigest()  # nosec - api requirement
        body = {"msgtype": "image", "image": {"base64": b64, "md5": md5}}
        return await self._send(body)

    async def send_news(self, articles: list[Mapping[str, Any]]) -> Dict[str, Any]:
        body = {"msgtype": "news", "news": {"articles": articles}}
        return await self._send(body)

    async def upload_media(self, file_bytes: bytes, filename: str, *, type_: str = "file") -> Dict[str, Any]:
        if type_ not in {"file", "voice"}:
            raise ValueError("type_ must be 'file' or 'voice'")

        url = _make_url(self.base_url, UPLOAD_PATH, self.key) + f"&type={type_}"

        # copy default headers but remove Content-Type so httpx can set the multipart boundary
        headers = dict(self._headers)
        headers.pop("Content-Type", None)

        file_length = len(file_bytes)

        if os.getenv("QWSEND_WEBHOOK_KEY"):
            boundary = "---------------------------" + uuid.uuid4().hex
            pre = (
                f"--{boundary}\r\n"
                f"Content-Disposition: form-data; name=\"media\"; filename=\"{filename}\"; filelength={file_length}\r\n"
                f"Content-Type: application/octet-stream\r\n\r\n"
            ).encode("utf-8")
            ending = f"\r\n--{boundary}--\r\n".encode("utf-8")
            body = pre + file_bytes + ending
            headers2 = dict(headers)
            headers2["Content-Type"] = f"multipart/form-data; boundary={boundary}"
            resp = await self._client.post(url, content=body, headers=headers2)
        else:
            files = {"media": (filename, io.BytesIO(file_bytes), "application/octet-stream")}
            resp = await self._client.post(url, files=files, headers=headers)

        return _ensure_ok(resp)
        

    async def send_file(self, media_id: str) -> Dict[str, Any]:
        body = {"msgtype": "file", "file": {"media_id": media_id}}
        return await self._send(body)

    async def send_voice(self, media_id: str) -> Dict[str, Any]:
        body = {"msgtype": "voice", "voice": {"media_id": media_id}}
        return await self._send(body)

    async def send_template_card(self, template_card: Mapping[str, Any]) -> Dict[str, Any]:
        body = {"msgtype": "template_card", "template_card": template_card}
        return await self._send(body)

    async def aclose(self) -> None:
        await self._client.aclose()
